import React, { memo, useMemo, useCallback } from 'react';
import { classNames } from '@shopify/css-utilities';
import { useTranslate, useGeolocation } from '../AppContext';
import { Month, Weekday, getOrderedWeekdays, getWeeksForMonth, getNextDisplayYear, getNextDisplayMonth, getPreviousDisplayYear, getPreviousDisplayMonth, getWeekStartDay, isToday } from '../../utilities/dates';
import { Text } from '../Text';
import { Icon } from '../Icon';
import styles from './DatePicker.css';
/* Simplified ISO format (ISO 8601) */

export function DatePicker(_ref) {
  var month = _ref.month,
      year = _ref.year,
      onMonthChange = _ref.onMonthChange;
  var translate = useTranslate();
  var geolocation = useGeolocation();
  var weekStartDay = useMemo(function () {
    return getWeekStartDay(geolocation === null || geolocation === void 0 ? void 0 : geolocation.countryCode);
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [geolocation]);
  var now = new Date();
  var currentMonth = now.getMonth() === month && now.getFullYear() === year;
  var weekdaysMarkup = getOrderedWeekdays(weekStartDay).map(function (weekday) {
    return /*#__PURE__*/React.createElement(WeekdayUI, {
      key: weekday,
      day: Weekday[weekday],
      current: currentMonth && new Date().getDay() === weekday
    });
  });
  var weeks = useMemo(function () {
    return getWeeksForMonth(month, year, weekStartDay);
  }, [month, year, weekStartDay]);
  var weeksMarkup = weeks.map(function (week) {
    var index = week.map(function (date) {
      return date === null || date === void 0 ? void 0 : date.getDate();
    }).join('');
    return /*#__PURE__*/React.createElement("tr", {
      className: styles.Week,
      key: index
    }, week.map(function (date) {
      return /*#__PURE__*/React.createElement(Day, {
        date: date,
        key: date === null || date === void 0 ? void 0 : date.getDate()
      });
    }));
  });
  var previousYear = getPreviousDisplayYear(month, year);
  var previousMonth = getPreviousDisplayMonth(month);
  var showPreviousMonthLabel = translate('showPreviousMonth', {
    month: translate(Month[previousMonth].toLowerCase()),
    year: "".concat(previousYear)
  });
  var nextYear = getNextDisplayYear(month, year);
  var nextMonth = getNextDisplayMonth(month);
  var showNextMonthLabel = translate('showNextMonth', {
    month: translate(Month[nextMonth].toLowerCase()),
    year: "".concat(nextYear)
  });
  var handleMonthChange = useCallback(function (month, year) {
    if (!onMonthChange) {
      return;
    }

    onMonthChange(month, year);
  }, [onMonthChange]);
  var monthLabelClassName = classNames(styles.MonthLabel, currentMonth && styles['MonthLabel-current']);
  return /*#__PURE__*/React.createElement("div", {
    className: styles.DatePicker
  }, onMonthChange && /*#__PURE__*/React.createElement("div", {
    className: styles.Controls
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: function onClick() {
      return handleMonthChange(previousMonth, previousYear);
    },
    className: styles.ControlsPrevious,
    "aria-label": showPreviousMonthLabel
  }, /*#__PURE__*/React.createElement(Icon, {
    source: "arrowLeft",
    size: "default"
  })), /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: function onClick() {
      return handleMonthChange(nextMonth, nextYear);
    },
    className: styles.ControlsNext,
    "aria-label": showNextMonthLabel
  }, /*#__PURE__*/React.createElement(Icon, {
    source: "arrowRight",
    size: "default"
  }))), /*#__PURE__*/React.createElement("table", {
    role: "grid",
    className: styles.Month
  }, /*#__PURE__*/React.createElement("caption", {
    className: monthLabelClassName,
    "aria-live": "polite"
  }, translate(Month[month].toLowerCase()), " ", year), /*#__PURE__*/React.createElement("thead", null, /*#__PURE__*/React.createElement("tr", {
    className: styles.WeekDays
  }, weekdaysMarkup)), /*#__PURE__*/React.createElement("tbody", null, weeksMarkup)));
}
var WeekdayUI = /*#__PURE__*/memo(function Weekday(_ref2) {
  var day = _ref2.day,
      current = _ref2.current;
  var translate = useTranslate();
  var className = classNames(styles.Weekday);
  return /*#__PURE__*/React.createElement("th", {
    scope: "col",
    className: className
  }, /*#__PURE__*/React.createElement(Text, {
    role: {
      type: 'abbreviation',
      "for": translate(day.toLowerCase())
    },
    subdued: !current,
    emphasized: current,
    size: "small"
  }, translate("".concat(day.toLowerCase(), "Abbreviation"))));
});

function Day(_ref3) {
  var date = _ref3.date,
      disabled = _ref3.disabled,
      selected = _ref3.selected,
      inRange = _ref3.inRange;

  if (!date) {
    return /*#__PURE__*/React.createElement("td", null);
  }

  var label = date.getDate();
  var today = isToday(date);
  var className = classNames(styles.Day, today && styles['Day-today'], disabled && styles['Day-disabled'], selected && styles['Day-selected'], inRange && styles['Day-inRange']);
  return /*#__PURE__*/React.createElement("td", {
    className: styles.DayCell
  }, disabled ? /*#__PURE__*/React.createElement("span", {
    className: className
  }, label) : /*#__PURE__*/React.createElement("button", {
    type: "button",
    tabIndex: -1,
    className: className
  }, label));
}