import React from 'react';
import faker from 'faker';
import { errorId } from '../../utilities/errors';
import { mountWithContext } from '../../test-utilities';
import { Text } from '../Text';
import { Icon } from '../Icon';
import { InlineError } from '../InlineError';
import { createTheme } from '../Theme';
import { Form } from '../Form';
import { View } from '../View';
import { TextField } from './TextField';
var defaultProps = {
  id: 'text_field',
  label: 'First name'
};
describe('<TextField />', function () {
  it('renders an input field with basic props', function () {
    var id = faker.random.alphaNumeric();
    var name = faker.random.word();
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      id: id,
      name: name
    })));
    expect(textField).toContainReactComponent('input', {
      id: id,
      name: name
    });
  });
  it('renders an input field with a prefilled value when the value prop is provided', function () {
    var value = 'Shopifolk';
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      value: value
    })));
    expect(textField).toContainReactComponent('input', {
      value: value
    });
  });
  it('renders an error message when the error prop is provided', function () {
    var error = 'Enter a first name';
    var id = faker.random.alphaNumeric();
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      id: id,
      error: error
    })));
    expect(textField).toContainReactComponent(InlineError, {
      controlID: id,
      children: error
    });
  });
  it('renders prefix on focus only when label is inside', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      prefix: "$"
    })));
    expect(textField).not.toContainReactComponent('div', {
      children: '$'
    });
    textField.find('input').trigger('onFocus');
    expect(textField).toContainReactComponent('div', {
      children: '$'
    });
    expect(textField).toContainReactComponent('input', {
      'aria-labelledby': 'text_field-label text_field-prefix'
    });
  });
  it('renders prefix always when label is outside', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      prefix: "$"
    })), {
      theme: createTheme({
        textFields: {
          labelPosition: 'outside'
        }
      })
    });
    expect(textField).toContainReactComponent('div', {
      children: '$'
    });
  });
  it('renders suffix', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      suffix: "$"
    })));
    expect(textField).toContainReactComponent('div', {
      children: '$'
    });
  });
  it('renders icon', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      icon: "mobile"
    })));
    expect(textField).toContainReactComponent(Icon, {
      source: 'mobile'
    });
  });
  it('renders actions', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, defaultProps, /*#__PURE__*/React.createElement("button", null, "x")));
    expect(textField).toContainReactComponent('button', {
      children: 'x'
    });
  });
  it('renders a disabled input element when the disabled prop is true', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      disabled: true
    })));
    expect(textField).toContainReactComponent('input', {
      disabled: true
    });
  });
  it('renders a readonly input element when the readonly prop is true', function () {
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      readonly: true
    })));
    expect(textField).toContainReactComponent('input', {
      readOnly: true
    });
  });
  it('calls an onFocus callback when the input gains focus', function () {
    var onFocus = jest.fn();
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      onFocus: onFocus
    })));
    textField.find('input').trigger('onFocus');
    expect(onFocus).toHaveBeenCalled();
  });
  it('calls an onBlur callback when the input loses focus', function () {
    var onBlur = jest.fn();
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      onBlur: onBlur
    })));
    textField.find('input').trigger('onBlur', {
      currentTarget: {
        value: ''
      }
    });
    expect(onBlur).toHaveBeenCalled();
  });
  it('calls an onInput callback on input change with the current value', function () {
    var onInput = jest.fn();
    var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
      onInput: onInput
    })));
    var value = faker.random.alphaNumeric();
    textField.find('input').trigger('oninput', {
      currentTarget: {
        value: value
      }
    });
    expect(onInput).toHaveBeenCalledWith(value);
  });
  describe('type', function () {
    it('is applied to the input', function () {
      var type = 'email';
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        type: type
      })));
      expect(textField).toContainReactComponent('input', {
        type: type
      });
    });
    it('normalizes the telephone type to tel', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        type: "telephone"
      })));
      expect(textField).toContainReactComponent('input', {
        type: 'tel'
      });
    });
  });
  describe('form', function () {
    it('renders a form id when rendered in a nested form', function () {
      var testId = faker.random.word();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop
      }, /*#__PURE__*/React.createElement(Form, {
        id: testId,
        onSubmit: noop
      }, /*#__PURE__*/React.createElement(TextField, defaultProps), ",")));
      expect(textField).toContainReactComponent('input', {
        form: testId
      });
    });
    it('does not render a form id when rendered in a non-nested form', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop
      }, /*#__PURE__*/React.createElement(TextField, defaultProps), ","));
      expect(textField).toContainReactComponent('input', {
        form: undefined
      });
    });
  });
  describe('onChange', function () {
    it('does not call onChange when the value changes but is not committed, but updates the value in the input', function () {
      var onChange = jest.fn();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        value: "",
        onChange: onChange
      })));
      var value = faker.random.alphaNumeric();
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: value
        }
      });
      expect(onChange).not.toHaveBeenCalled();
      expect(textField).toContainReactComponent('input', {
        value: value
      });
    });
    it('does not call onChange when the value changes and is committed, but matches the current prop value', function () {
      var value = faker.random.alphaNumeric();
      var onChange = jest.fn();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        value: value,
        onChange: onChange
      })));
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: ''
        }
      });
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: value
        }
      });
      textField.find('input').trigger('onBlur', {
        currentTarget: {
          value: value
        }
      });
      expect(onChange).not.toHaveBeenCalled();
    });
    it('calls onChange when the value changes and is committed, and does not match the current prop value', function () {
      var onChange = jest.fn();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        value: "",
        onChange: onChange
      })));
      var value = faker.random.alphaNumeric();
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: value
        }
      });
      textField.find('input').trigger('onBlur', {
        currentTarget: {
          value: value
        }
      });
      expect(onChange).toHaveBeenCalledWith(value);
    });
    it('calls onChange when the TextField is focused and the enter key is pressed', function () {
      var onChange = jest.fn();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        value: "",
        onChange: onChange
      })));
      var value = faker.random.alphaNumeric();
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: value
        }
      });
      textField.find('input').trigger('onKeyDown', {
        key: 'Enter',
        currentTarget: {
          value: value
        }
      });
      expect(onChange).toHaveBeenCalledWith(value);
    });
  });
  describe('label', function () {
    it('renders a placeholder by default', function () {
      var label = 'Email';
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      expect(textField).toContainReactComponent('input', {
        placeholder: label
      });
    });
    it('hides the placeholder once a value is entered', function () {
      var label = 'Email';
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: faker.random.alphaNumeric()
        }
      });
      expect(textField).not.toContainReactComponent('input', {
        placeholder: label
      });
    });
    it('renders the placeholder once a value is entered and removed without focusing the field', function () {
      var label = 'Email';
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: 'snowdevil@shopify.com'
        }
      });
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: ''
        }
      });
      expect(textField).toContainReactComponent('input', {
        placeholder: label
      });
    });
    it('hides the placeholder once a value is entered and removed after the field loses focus', function () {
      var label = 'Email';
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      textField.find('input').trigger('oninput', {
        currentTarget: {
          value: 'snowdevil@shopify.com'
        }
      });
      textField.find('input').trigger('onBlur', {
        currentTarget: {
          value: ''
        }
      });
      expect(textField).not.toContainReactComponent('input', {
        placeholder: label
      });
    });
    it.todo('calls `labelled.onFocus when focusing the textfield`');
    it.todo('calls `labelled.onBlur when removing focus from the textfield`');
    it.todo('calls `labelled.onChange with a value when typing a value in the field`');
    it.todo('calls `labelled.onChange with an empty value when typing and the field is empty`');
  });
  describe('autofocus', function () {
    var rawCreateElement = document.createElement;
    var focusSpy = jest.fn();
    var createElementSpy = jest.spyOn(document, 'createElement');
    beforeEach(function () {
      createElementSpy.mockImplementation(function () {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        var result = rawCreateElement.apply(document, args);

        if (args[0].toLocaleLowerCase() === 'input') {
          result.focus = focusSpy;
        }

        return result;
      });
    });
    afterEach(function () {
      focusSpy.mockReset();
    });
    afterAll(function () {
      createElementSpy.mockRestore();
    });
    it('does not focus inputs by default', function () {
      mountWithContext( /*#__PURE__*/React.createElement(TextField, defaultProps));
      expect(focusSpy).not.toHaveBeenCalled();
    });
    it('focuses on the input when autofocus is specified', function () {
      mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        autofocus: true
      })));
      expect(focusSpy).toHaveBeenCalledTimes(1);
    });
  });
  describe('maxLength', function () {
    it('renders an input field with maxLength', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        maxLength: 10
      })));
      expect(textField).toContainReactComponent('input', {
        maxLength: 10
      });
      expect(textField).not.toContainReactComponent('textarea');
    });
    it('renders a textarea with maxLength', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        multiline: true,
        maxLength: 10
      })));
      expect(textField).toContainReactComponent('textarea', {
        maxLength: 10
      });
      expect(textField).not.toContainReactComponent('input');
    });
  });
  describe('controlledValue', function () {
    it('is preferred as the value when provided', function () {
      var value = faker.random.words();
      var controlledValue = faker.random.words();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        value: value,
        controlledValue: controlledValue
      })));
      expect(textField).toContainReactComponent('input', {
        value: controlledValue
      });
    });
  });
  describe('accessibilityDescription', function () {
    it('omits the hidden description element when accessibilityDescription is not specified', function () {
      var id = faker.random.alphaNumeric();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        id: id
      })));
      expect(textField).not.toContainReactComponent(Text, {
        id: "".concat(id, "-description")
      });
    });
    it('renders a hidden element containing accessibilityDescription text', function () {
      var id = faker.random.alphaNumeric();
      var description = faker.random.words();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        id: id,
        accessibilityDescription: description
      })));
      var descriptionElement = textField.find(View, {
        id: "".concat(id, "-description")
      });
      expect(descriptionElement).toContainReactText(description);
    });
    it('associates the input with the hidden accessibilityDescription element', function () {
      var id = faker.random.alphaNumeric();
      var description = faker.random.words();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        id: id,
        accessibilityDescription: description
      })));
      expect(textField).toContainReactComponent('input', {
        id: id,
        'aria-describedby': "".concat(id, "-description")
      });
    });
    it('includes error element id in describedy associations', function () {
      var id = faker.random.alphaNumeric();
      var description = faker.random.words();
      var error = faker.random.words();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        id: id,
        accessibilityDescription: description,
        error: error
      })));
      expect(textField).toContainReactComponent('input', {
        id: id,
        'aria-describedby': "".concat(id, "-description ").concat(errorId(id))
      });
    });
  });
  describe('multiline', function () {
    it('renders a textarea when the multiline prop is provided', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        multiline: true
      })));
      expect(textField).toContainReactComponent('textarea');
      expect(textField).not.toContainReactComponent('input');
    });
    it('renders a textarea with rows when the multiline prop is a number', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        multiline: 5
      })));
      expect(textField).toContainReactComponent('textarea', {
        rows: 5
      });
    });
    it('does not render a textarea with rows when the multiline prop is a 0', function () {
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        multiline: 0
      })));
      expect(textField).not.toContainReactComponent('textarea', {
        rows: 0
      });
    });
  });
  describe('forward refs', function () {
    it('passes input element to ref functions', function () {
      var id = faker.random.alphaNumeric();
      var refCallback = jest.fn();
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        id: id,
        ref: refCallback
      })));
      expect(refCallback).toHaveBeenCalledWith(textField.find('input').instance);
    });
    it('assigns input element to ref.current on ref instances', function () {
      var id = faker.random.alphaNumeric();
      var fakeRef = {
        current: null
      };
      var textField = mountWithContext( /*#__PURE__*/React.createElement(TextField, Object.assign({}, defaultProps, {
        id: id,
        ref: fakeRef
      })));
      expect(fakeRef.current).toBe(textField.find('input').instance);
    });
  });
});

function noop() {}