"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Tabs = Tabs;

var _react = _interopRequireWildcard(require("react"));

var _cssUtilities = require("@shopify/css-utilities");

var _Icon = require("../Icon");

var _InlineStack = require("../InlineStack");

var _id = require("../../utilities/id");

var _Tabs = _interopRequireDefault(require("./Tabs.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const createId = (0, _id.createIdCreator)('Tabs');
const TabsContext = /*#__PURE__*/(0, _react.createContext)(null);
/**
 * Tabs are used when there’s multiple ways to display the same information,
 * like a map or list view. When a buyer needs to make a choice, use the
 * OptionList component instead as Buyers are more likely to choose the
 * non-default option. Don’t hide important information in tabs. Limit to two
 * of three tabs and keep labels short to optimize for mobile.
 */

function Tabs({
  tabs,
  selected,
  ariaLabel,
  bordered,
  onChange,
  children
}) {
  const id = (0, _id.useId)(undefined, createId);
  const tabList = (0, _react.useRef)(null);
  const lastIndex = tabs.length - 1;

  const childrenCount = _react.Children.count(children);

  const handleKeyDown = (0, _react.useCallback)(event => {
    const prev = selected - 1 < 0 ? lastIndex : selected - 1;
    const next = selected + 1 > lastIndex ? 0 : selected + 1;
    const handlerMap = new Map([['ArrowLeft', prev], ['Left', prev], ['ArrowRight', next], ['Right', next], ['Home', 0], ['End', lastIndex]]);
    const newSelected = handlerMap.get(event.key);

    if (newSelected != null) {
      event.preventDefault();
      onChange(newSelected);
    }
  }, [onChange, selected, lastIndex]);
  (0, _react.useEffect)(() => {
    const {
      current
    } = tabList;
    if (current == null) return;
    current.addEventListener('keydown', handleKeyDown, false);
    return () => {
      current.removeEventListener('keydown', handleKeyDown, false);
    };
  }, [tabList, handleKeyDown]);
  (0, _react.useEffect)(() => {
    if (childrenCount !== tabs.length) {
      // eslint-disable-next-line no-console
      console.warn(`<Tabs /> has ${tabs.length} tabs and ${childrenCount} children. A mismatch will cause some content to be unavailable.`);
    }
  }, [childrenCount, tabs.length]);
  const tabsMarkup = [];
  const panelsMarkup = [];

  _react.Children.forEach(children, (child, index) => {
    const tab = tabs[index];

    if (tab != null) {
      const {
        label,
        icon
      } = typeof tab === 'string' ? {
        label: tab,
        icon: undefined
      } : tab;
      tabsMarkup.push( /*#__PURE__*/_react.default.createElement(Tab, Object.assign({
        label,
        icon,
        selected,
        onChange,
        id
      }, {
        index: index,
        key: label
      })));
    }

    panelsMarkup.push( /*#__PURE__*/_react.default.createElement(TabPanel, {
      index: index
    }, child));
  });

  return /*#__PURE__*/_react.default.createElement(TabsContext.Provider, {
    value: {
      id,
      selected,
      bordered
    }
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: _Tabs.default.Tabs
  }, /*#__PURE__*/_react.default.createElement("div", {
    ref: tabList,
    role: "tablist",
    "aria-label": ariaLabel,
    className: _Tabs.default.TabList
  }, tabsMarkup), panelsMarkup));
}

function Tab({
  label,
  icon,
  index,
  selected: selectedIndex,
  onChange,
  id
}) {
  if (index == null) {
    throw new Error('`index` is missing');
  }

  const tabRef = (0, _react.useRef)(null);
  const mountedRef = (0, _react.useRef)(false);
  const selected = selectedIndex === index;
  const markup = icon == null ? label : /*#__PURE__*/_react.default.createElement(_InlineStack.InlineStack, {
    spacing: "extraTight",
    blockAlignment: "center"
  }, /*#__PURE__*/_react.default.createElement(_Icon.Icon, {
    source: icon,
    size: "large"
  }), /*#__PURE__*/_react.default.createElement("span", null, label));
  (0, _react.useEffect)(() => {
    if (tabRef.current && selected && mountedRef.current) tabRef.current.focus();else mountedRef.current = true;
  }, [tabRef, selected]);
  return /*#__PURE__*/_react.default.createElement("button", {
    type: "button",
    role: "tab",
    tabIndex: selected ? 0 : -1,
    "aria-selected": selected,
    id: `${id}Tab${index}`,
    "aria-controls": `${id}TabPanel${index}`,
    onClick: () => onChange(index),
    className: (0, _cssUtilities.classNames)(_Tabs.default.Tab, selected && _Tabs.default.selected),
    ref: tabRef
  }, markup);
}

function TabPanel({
  index,
  children
}) {
  const {
    selected: selectedIndex,
    id,
    bordered
  } = useTabs();
  const selected = selectedIndex === index;
  return /*#__PURE__*/_react.default.createElement("div", {
    role: "tabpanel",
    id: `${id}TabPanel${index}`,
    "aria-labelledby": `${id}Tab${index}`,
    hidden: !selected,
    className: (0, _cssUtilities.classNames)(_Tabs.default.TabPanel, bordered && _Tabs.default.bordered)
  }, children);
}

function useTabs() {
  const context = (0, _react.useContext)(TabsContext);

  if (context == null) {
    throw new Error('No tabs found in context');
  }

  return context;
}