"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const faker_1 = __importDefault(require("faker"));
const test_utilities_1 = require("../../test-utilities");
const Portal_1 = require("../Portal");
const Heading_1 = require("../Heading");
const Modal_1 = require("./Modal");
describe('<Modal />', () => {
    const defaultIFrameSrc = faker_1.default.internet.url();
    describe('open prop', () => {
        it('renders the modal when the open prop is true', async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal open src={defaultIFrameSrc}/>);
            expect(modal).toContainReactComponent(Portal_1.Portal);
        });
        it("doesn't render the modal when the open prop is undefined", async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc}/>);
            expect(modal).not.toContainReactComponent(Portal_1.Portal);
        });
        it('calls onClose when open changes from true to false', async () => {
            const onCloseSpy = jest.fn();
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal onClose={onCloseSpy} src={defaultIFrameSrc} open/>);
            modal.setProps({ open: false });
            expect(onCloseSpy).toHaveBeenCalledTimes(1);
        });
    });
    it('renders an iframe is the src prop is passed', async () => {
        const iFrameSrc = faker_1.default.internet.url();
        const iFrameName = faker_1.default.random.words(2);
        const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={iFrameSrc} iFrameName={iFrameName} open/>);
        expect(modal).toContainReactComponent('iframe', {
            src: iFrameSrc,
            name: iFrameName,
        });
    });
    it('renders its children if no src prop is passed', async () => {
        const content = faker_1.default.random.words(2);
        const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal open>
        <p>{content}</p>
      </Modal_1.Modal>);
        expect(modal).not.toContainReactComponent('iframe');
        expect(modal).toContainReactText(content);
    });
    it('renders a header with Heading if the title prop is passed', async () => {
        const title = faker_1.default.random.words(2);
        const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} title={title} open/>);
        expect(modal).toContainReactComponent('header');
        expect(modal).toContainReactComponent(Heading_1.Heading);
        expect(modal).toContainReactText(title);
    });
    it('does not render a Heading if titleHidden', async () => {
        const title = faker_1.default.random.words(2);
        const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} title={title} open titleHidden/>);
        expect(modal).not.toContainReactComponent(Heading_1.Heading);
    });
    it('still renders a close button if no title', async () => {
        const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} open/>);
        expect(modal).toContainReactComponent('button');
    });
    it('still renders a close button if titleHidden', async () => {
        const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} open titleHidden/>);
        expect(modal).toContainReactComponent('button');
    });
    describe('when the escape key is pressed', () => {
        it('closes the modal', async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} open/>);
            const event = new KeyboardEvent('keydown', { key: 'Escape' });
            modal.act(() => document.dispatchEvent(event));
            expect(modal).not.toContainReactComponent(Portal_1.Portal);
        });
        it('calls onClose callback', async () => {
            const onCloseSpy = jest.fn();
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal onClose={onCloseSpy} open>
          Hi
        </Modal_1.Modal>);
            const event = new KeyboardEvent('keydown', { key: 'Escape' });
            modal.act(() => document.dispatchEvent(event));
            expect(onCloseSpy).toHaveBeenCalledTimes(1);
        });
    });
    describe('when the close button is clicked', () => {
        it('closes the modal', async () => {
            var _a;
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} title="A modal" open/>);
            (_a = modal.find('button')) === null || _a === void 0 ? void 0 : _a.trigger('onClick');
            expect(modal).not.toContainReactComponent(Portal_1.Portal);
        });
        it('calls onClose callback', async () => {
            var _a;
            const onCloseSpy = jest.fn();
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal title="A modal" onClose={onCloseSpy} open>
          Hi
        </Modal_1.Modal>);
            (_a = modal.find('button')) === null || _a === void 0 ? void 0 : _a.trigger('onClick');
            expect(onCloseSpy).toHaveBeenCalledTimes(1);
        });
        it('resets the iFrameHeight', async () => {
            var _a, _b;
            const expectedHeight = 100;
            const fakeEvent = {
                target: {
                    contentWindow: { document: { body: { scrollHeight: expectedHeight } } },
                },
            };
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal title="A modal" src={defaultIFrameSrc} open/>);
            (_a = modal.find('iframe')) === null || _a === void 0 ? void 0 : _a.trigger('onLoad', fakeEvent);
            expect(modal.find('iframe').prop('style')).toMatchObject({
                height: `${expectedHeight}px`,
            });
            (_b = modal.find('button')) === null || _b === void 0 ? void 0 : _b.trigger('onClick');
            modal.setProps({ open: false });
            modal.setProps({ open: true });
            expect(modal.find('iframe').prop('style')).toBeUndefined();
        });
    });
    describe('blocking prop', () => {
        it("doesn't render the close button", async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} title="A modal" open blocking/>);
            expect(modal).not.toContainReactComponent('button');
        });
        it("doesn't close the modal when the escape key is pressed", async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} open blocking/>);
            const event = new KeyboardEvent('keydown', { key: 'Escape' });
            modal.act(() => document.dispatchEvent(event));
            expect(modal).toContainReactComponent(Portal_1.Portal);
        });
    });
    describe('long prop', () => {
        it("doesn't add the onLoad prop when present", async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} open long/>);
            const iframe = modal.find('iframe');
            expect(iframe === null || iframe === void 0 ? void 0 : iframe.prop('onLoad')).toBeUndefined();
        });
        it('adds the onLoad prop when absent', async () => {
            const modal = await test_utilities_1.mountWithContext(<Modal_1.Modal src={defaultIFrameSrc} open/>);
            const iframe = modal.find('iframe');
            expect(iframe === null || iframe === void 0 ? void 0 : iframe.prop('onLoad')).not.toBeUndefined();
        });
    });
});
