"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const faker_1 = __importDefault(require("faker"));
const test_utilities_1 = require("../../../../test-utilities");
const Checkbox_1 = require("../../../Checkbox");
const RadioControl_1 = require("../../../RadioControl");
const View_1 = require("../../../View");
const OptionList_1 = require("../../OptionList");
const Option_1 = require("./Option");
const defaultProps = {
    id: faker_1.default.random.uuid(),
    label: faker_1.default.random.word(),
};
const defaultOptionListProps = {
    id: faker_1.default.random.uuid(),
    selectedItems: [],
    onChange: () => null,
    children: <div />,
};
describe('<Option />', () => {
    it('triggers onChange when input is selected', () => {
        const id = faker_1.default.random.uuid();
        const onChangeSpy = jest.fn();
        const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps} onChange={onChangeSpy}>
        <Option_1.Option {...defaultProps} id={id}/>
      </OptionList_1.OptionList>);
        optionList.find(Option_1.Option).find(RadioControl_1.RadioControl).trigger('onChange');
        expect(onChangeSpy).toHaveBeenCalledWith([id]);
    });
    it('input is active if selected', () => {
        const id = faker_1.default.random.uuid();
        const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps} selectedItems={[id]}>
        <Option_1.Option {...defaultProps} id={id}/>
      </OptionList_1.OptionList>);
        expect(optionList.find(Option_1.Option)).toContainReactComponent('input', {
            checked: true,
        });
    });
    it('renders a RadioControl if allow multiple is false', () => {
        const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps}>
        <Option_1.Option {...defaultProps}/>
      </OptionList_1.OptionList>);
        expect(optionList.find(Option_1.Option)).toContainReactComponent(RadioControl_1.RadioControl);
    });
    it('renders a CheckboxControl if allow multiple is true', () => {
        const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps} allowMultiple>
        <Option_1.Option {...defaultProps}/>
      </OptionList_1.OptionList>);
        expect(optionList.find(Option_1.Option)).toContainReactComponent(Checkbox_1.CheckboxControl);
    });
    it('renders visually hidden View when label and childrens are set', () => {
        const labelContent = faker_1.default.random.word();
        const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps}>
        <Option_1.Option {...defaultProps} label={labelContent}>
          Childrens
        </Option_1.Option>
      </OptionList_1.OptionList>);
        expect(optionList.find(View_1.View, { visibility: 'hidden' })).toContainReactText(labelContent);
    });
    describe('allowMultiple', () => {
        it('allows the selection of multiple options', () => {
            const firstItem = faker_1.default.random.uuid();
            const secondItem = faker_1.default.random.uuid();
            const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps} allowMultiple selectedItems={[firstItem, secondItem]}>
          <Option_1.Option {...defaultProps} id={firstItem}/>
          <Option_1.Option {...defaultProps} id={secondItem}/>
        </OptionList_1.OptionList>);
            const [firstOption, secondOption] = optionList.findAll(Option_1.Option);
            expect(firstOption).toContainReactComponent(Checkbox_1.CheckboxControl, {
                checked: true,
            });
            expect(secondOption).toContainReactComponent(Checkbox_1.CheckboxControl, {
                checked: true,
            });
        });
        it('toggles the selection of already selection options', () => {
            const selected = faker_1.default.random.uuid();
            const onChangeSpy = jest.fn();
            const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps} allowMultiple selectedItems={[selected]} onChange={onChangeSpy}>
          <Option_1.Option {...defaultProps} id={selected}/>
        </OptionList_1.OptionList>);
            optionList.find(Option_1.Option).find(Checkbox_1.CheckboxControl).trigger('onChange');
            expect(onChangeSpy).toHaveBeenCalledWith([]);
        });
    });
    describe('controlHidden', () => {
        it('shows control by default', () => {
            const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps}>
          <Option_1.Option {...defaultProps}/>
        </OptionList_1.OptionList>);
            expect(optionList.find(Option_1.Option)).toContainReactComponent(RadioControl_1.RadioControl);
        });
        it('hides control when true', () => {
            const optionList = test_utilities_1.mountWithContext(<OptionList_1.OptionList {...defaultOptionListProps} controlHidden>
          <Option_1.Option {...defaultProps}/>
        </OptionList_1.OptionList>);
            expect(optionList.find(Option_1.Option)).not.toContainReactComponent(RadioControl_1.RadioControl);
        });
    });
});
